#!/usr/bin/env python

# render_forest Rel 4
# This script runs steps described here -> https://www.cartographersguild.com/showthread.php?t=10009
# Minus the individual trees, we just want a general forest
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimplearn.com or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: less dense forest option.
# Rel 3: more improvements to trees/forest
# Rel 4: allow user to select custom pattern instead of using dense or not desnse forest pattern.

import random
from gimpfu import *

def python_tt_render_forest(image, layer, dense, custom, pattern):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	seed =  random.randrange(100000,10000000) #ocean and land
	forest_start = pdb.gimp_image_get_layer_by_name(image,"Forest Start")
	pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,forest_start,(0,0,0))
	forest_outline_1 = pdb.gimp_selection_save(image)
	pdb.gimp_selection_none(image)
	pdb.gimp_item_set_name(forest_outline_1,"Forest outline 1")
	pdb.gimp_item_set_visible(forest_start,False)
	#Forest
	forest = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Forest",100,LAYER_MODE_NORMAL_LEGACY)
	pdb.gimp_image_insert_layer(image,forest,None,0)
	if custom == 0:
		if dense == 1:
			pdb.gimp_context_set_pattern("Small Sized Forests")
		else:
			pdb.gimp_context_set_pattern("Trees Black & Whte 2")
	else:
		pdb.gimp_context_set_pattern(pattern)
	pdb.gimp_edit_fill(forest,FILL_PATTERN)
	
	#Forest Copy 
	forest_copy = pdb.gimp_layer_new_from_drawable(forest,image)
	pdb.gimp_image_insert_layer(image,forest_copy,None,0)
	
	#Forest Frame
	forest_frame = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Forest Frame",100,LAYER_MODE_NORMAL_LEGACY)
	pdb.gimp_image_insert_layer(image,forest_frame,None,0)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(forest_frame,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_1)
	pdb.gimp_edit_fill(forest_frame,FILL_WHITE)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,forest_frame,50,50,1)
	pdb.gimp_layer_set_mode(forest_frame,LAYER_MODE_MULTIPLY_LEGACY)
	
	#merge down, threshold and save selection 
	forest_copy = pdb.gimp_image_merge_down(image,forest_frame,EXPAND_AS_NECESSARY)
	pdb.gimp_threshold(forest_copy,63,255)
	pdb.gimp_by_color_select_full(forest_copy,(255,255,255),0,CHANNEL_OP_REPLACE,True,False,0,0,True,True,SELECT_CRITERION_COMPOSITE)
	forest_outline_2 = pdb.gimp_selection_save(image)
	
	pdb.gimp_layer_set_mode(forest_copy,LAYER_MODE_MULTIPLY_LEGACY)
	#merge down to forest
	forest = pdb.gimp_image_merge_down(image,forest_copy,EXPAND_AS_NECESSARY)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(forest)
	pdb.gimp_selection_none(image)
	pdb.plug_in_hsv_noise(image,forest,2,0,0,30)
	#Tree Bumps
	tree_bumps = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Tree bumps",100,LAYER_MODE_NORMAL_LEGACY)
	pdb.gimp_image_insert_layer(image,tree_bumps,None,0)
	pdb.gimp_context_set_foreground((128,128,128))
	pdb.gimp_edit_fill(tree_bumps,FILL_FOREGROUND)
	pdb.plug_in_bump_map(image,tree_bumps,forest,120,30,2,0,0,0,0,True,False,0)
	
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_2)
	tree_bumps_mask = pdb.gimp_layer_create_mask(tree_bumps,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(tree_bumps,tree_bumps_mask)
	pdb.gimp_selection_none(image)
	
	#color 1
	color_1 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Tree Color 1",100,LAYER_MODE_SOFTLIGHT_LEGACY)
	pdb.gimp_image_insert_layer(image,color_1,None,0)
	pdb.gimp_context_set_foreground((29,35,0))
	pdb.gimp_edit_fill(color_1,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_2)
	color_1_mask = pdb.gimp_layer_create_mask(color_1,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(color_1,color_1_mask)
	pdb.gimp_selection_none(image)
	
	#color 2
	color_2 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Tree Color 2",100,LAYER_MODE_MULTIPLY_LEGACY)
	pdb.gimp_image_insert_layer(image,color_2,None,0)
	pdb.gimp_context_set_foreground((59,88,14))
	pdb.gimp_edit_fill(color_2,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_2)
	color_2_mask = pdb.gimp_layer_create_mask(color_2,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(color_2,color_2_mask)
	pdb.gimp_selection_none(image)
	
	#color 3
	color_3 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Tree Color 3",100,LAYER_MODE_SOFTLIGHT_LEGACY)
	pdb.gimp_image_insert_layer(image,color_3,None,0)
	pdb.gimp_context_set_foreground((134,159,48))
	pdb.gimp_edit_fill(color_3,FILL_FOREGROUND)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_2)
	color_3_mask = pdb.gimp_layer_create_mask(color_3,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(color_3,color_3_mask)
	pdb.gimp_selection_none(image)
	
	#Trees Noise
	tree_noise = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Tree Noise",100,LAYER_MODE_OVERLAY_LEGACY)
	pdb.gimp_image_insert_layer(image,tree_noise,None,0)
	pdb.plug_in_solid_noise(image,tree_noise,False,False,seed,15,16,16)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,forest_outline_2)
	tree_noise_mask = pdb.gimp_layer_create_mask(tree_noise,ADD_MASK_SELECTION)
	pdb.gimp_layer_add_mask(tree_noise,tree_noise_mask)
	pdb.gimp_selection_none(image)
	
	#5. Hide "Forest" layer. Gauss blur 5 and Noise Spread 5 the Mask of "Tree Noise" and copy this mask to 
    # "Tree Color 1", "Tree Color 2", "Tree Color 3" and "Tree bumps".
	pdb.gimp_item_set_visible(forest, False)
	pdb.plug_in_gauss(image,tree_noise_mask,5,5,1)
	pdb.plug_in_spread(image,tree_noise_mask,5,5)
	pdb.gimp_edit_copy(tree_noise_mask)
	floating_sel = pdb.gimp_edit_paste(color_3_mask,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	floating_sel = pdb.gimp_edit_paste(color_2_mask,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	floating_sel = pdb.gimp_edit_paste(color_1_mask,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	floating_sel = pdb.gimp_edit_paste(tree_bumps_mask,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	if dense == 0:
		tree_bumps_2 = pdb.gimp_layer_new_from_drawable(tree_bumps,image)
		pdb.gimp_image_insert_layer(image,tree_bumps_2,None,0)
		pdb.gimp_item_set_name(tree_bumps_2,"Tree bumps 2 for less dense forest")
		pdb.gimp_layer_set_opacity(tree_bumps_2,50)
		pdb.gimp_layer_set_mode(tree_bumps_2,LAYER_MODE_OVERLAY_LEGACY)
	
	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_render_forest",                           
	"Render D&D Forest\nOn a transparent layer, use pure black to draw\nwhere you want forest(s) to generally be at.\nName the layer 'Forest Start'.",
	"Render D&D Forest",
	"Tin Tran",
	"Tin Tran",
	"July 2017",
	"<Image>/Filters/Render/Render Forest...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_TOGGLE, "dense",   "Dense Forest:", 1),
	(PF_TOGGLE, "custom",   "Use Custom Pattern (below) Instead:", 0),
	(PF_PATTERN, "pattern", "Custom Pattern:", 0),
	#(PF_SPINNER, "blur_radius", "Guasian Blur Radius:", 33, (0, 12800, 0.1)),  # alias PF_ADJUSTMENT
	],
	[],
	python_tt_render_forest)

main()
